@@ -8,10 +8,28 @@
 
 #ifndef DWARF_WRAPPERS_H
 #define DWARF_WRAPPERS_H 1
+
+#include "config.h"
+
+extern "C" {
 #include <elfutils/libdw.h>
+#ifdef HAVE_ELFUTILS_VERSION_H
+#include <elfutils/version.h>
+#endif
+}
 
 #include <string>
 
+#if !defined(_ELFUTILS_PREREQ)
+// make a dummy PREREQ check for elfutils < 0.138
+#define _ELFUTILS_PREREQ(major, minor) (0 >= 1)
+#endif
+
+#if ! _ELFUTILS_PREREQ(0,142)
+#define DW_TAG_rvalue_reference_type 0x42
+#endif
+
+
 // NB: "rc == 0" means OK in this case
 void dwfl_assert(const std::string& desc, int rc);
 
@@ -89,7 +107,35 @@ public:
   }
 };
 
-const char *dwarf_diename_integrate (Dwarf_Die *die);
+
+// Look up the DIE for a reference-form attribute name
+inline Dwarf_Die *
+dwarf_attr_die (Dwarf_Die *die, unsigned int attr, Dwarf_Die *result)
+{
+  Dwarf_Attribute attr_mem;
+  return dwarf_formref_die (dwarf_attr_integrate (die, attr, &attr_mem),
+                            result);
+}
+
+
+#if !_ELFUTILS_PREREQ(0, 143)
+// Elfutils prior to 0.143 didn't use attr_integrate when looking up the
+// decl_file or decl_line, so the attributes would sometimes be missed.  For
+// those old versions, we define custom implementations to do the integration.
+
+const char *dwarf_decl_file_integrate (Dwarf_Die *die);
+#define dwarf_decl_file dwarf_decl_file_integrate
+
+int dwarf_decl_line_integrate (Dwarf_Die *die, int *linep)
+  __nonnull_attribute__ (2);
+#define dwarf_decl_line dwarf_decl_line_integrate
+
+#endif // !_ELFUTILS_PREREQ(0, 143)
+
+
+// Resolve a full name for dwarf types
+bool dwarf_type_name(Dwarf_Die *type_die, std::string& type_name);
+std::string dwarf_type_name(Dwarf_Die *type_die);
 
 
 #endif
