@@ -1,5 +1,7 @@
 #include <string>
 #include <vector>
+#include <sstream>
+#include <fstream>
 
 extern "C" {
 #include <string.h>
@@ -13,9 +15,11 @@ class hash
 {
 private:
   struct mdfour md4;
+  std::ostringstream parm_stream;
 
 public:
   hash() { start(); }
+  hash(const hash &base) { md4 = base.md4; parm_stream << base.parm_stream.str(); }
 
   void start();
 
@@ -34,9 +38,11 @@ public:
   void add_file(const std::string& filename);
 
   void result(std::string& r);
+  std::string get_parms();
 };
 
-void find_hash (systemtap_session& s, const std::string& script);
+void find_script_hash (systemtap_session& s, const std::string& script);
+void find_stapconf_hash (systemtap_session& s);
 std::string find_tracequery_hash (systemtap_session& s,
                                   const std::vector<std::string>& headers);
 std::string find_typequery_hash (systemtap_session& s, const std::string& name);
