@@ -80,6 +80,16 @@
 
 #define STAP_UPROBE_GUARD 0x31425250
 
+#ifndef STAP_SDT_VOLATILE /* allow users to override */
+#if (__GNUC__ >= 4 && __GNUC_MINOR__ >= 5 \
+     || (defined __GNUC_RH_RELEASE__ \
+         && __GNUC__ == 4 && __GNUC_MINOR__ == 4 && __GNUC_PATCHLEVEL__ >= 3 \
+         && (__GNUC_PATCHLEVEL__ > 3 || __GNUC_RH_RELEASE__ >= 10)))
+#define STAP_SDT_VOLATILE
+#else
+#define STAP_SDT_VOLATILE volatile
+#endif
+#endif
 #define STAP_PROBE_(probe)			\
 do {						\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);	\
@@ -89,7 +99,7 @@ do {						\
 
 #define STAP_PROBE1_(probe,label,parm1)			\
 do STAP_SEMAPHORE(probe) {				\
-  volatile __typeof__((parm1)) arg1 = parm1;		\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;	\
   STAP_UNINLINE;					\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);		\
   __asm__ volatile ("2:\n"				\
@@ -98,8 +108,8 @@ do STAP_SEMAPHORE(probe) {				\
 
 #define STAP_PROBE2_(probe,label,parm1,parm2)				\
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -108,9 +118,9 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE3_(probe,label,parm1,parm2,parm3)			\
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -119,10 +129,10 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE4_(probe,label,parm1,parm2,parm3,parm4)		\
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -131,11 +141,11 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE5_(probe,label,parm1,parm2,parm3,parm4,parm5)		\
 do  STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -144,12 +154,12 @@ do  STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE6_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6)	\
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
-  volatile __typeof__((parm6)) arg6 = parm6;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
+  STAP_SDT_VOLATILE __typeof__((parm6)) arg6 = parm6;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -158,13 +168,13 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE7_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7) \
 do  STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
-  volatile __typeof__((parm6)) arg6 = parm6;				\
-  volatile __typeof__((parm7)) arg7 = parm7;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
+  STAP_SDT_VOLATILE __typeof__((parm6)) arg6 = parm6;			\
+  STAP_SDT_VOLATILE __typeof__((parm7)) arg7 = parm7;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -173,14 +183,14 @@ do  STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE8_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8) \
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
-  volatile __typeof__((parm6)) arg6 = parm6;				\
-  volatile __typeof__((parm7)) arg7 = parm7;				\
-  volatile __typeof__((parm8)) arg8 = parm8;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
+  STAP_SDT_VOLATILE __typeof__((parm6)) arg6 = parm6;			\
+  STAP_SDT_VOLATILE __typeof__((parm7)) arg7 = parm7;			\
+  STAP_SDT_VOLATILE __typeof__((parm8)) arg8 = parm8;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -189,15 +199,15 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE9_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9) \
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
-  volatile __typeof__((parm6)) arg6 = parm6;				\
-  volatile __typeof__((parm7)) arg7 = parm7;				\
-  volatile __typeof__((parm8)) arg8 = parm8;				\
-  volatile __typeof__((parm9)) arg9 = parm9;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
+  STAP_SDT_VOLATILE __typeof__((parm6)) arg6 = parm6;			\
+  STAP_SDT_VOLATILE __typeof__((parm7)) arg7 = parm7;			\
+  STAP_SDT_VOLATILE __typeof__((parm8)) arg8 = parm8;			\
+  STAP_SDT_VOLATILE __typeof__((parm9)) arg9 = parm9;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
@@ -206,16 +216,16 @@ do STAP_SEMAPHORE(probe) {						\
 
 #define STAP_PROBE10_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9,parm10) \
 do STAP_SEMAPHORE(probe) {						\
-  volatile __typeof__((parm1)) arg1 = parm1;				\
-  volatile __typeof__((parm2)) arg2 = parm2;				\
-  volatile __typeof__((parm3)) arg3 = parm3;				\
-  volatile __typeof__((parm4)) arg4 = parm4;				\
-  volatile __typeof__((parm5)) arg5 = parm5;				\
-  volatile __typeof__((parm6)) arg6 = parm6;				\
-  volatile __typeof__((parm7)) arg7 = parm7;				\
-  volatile __typeof__((parm8)) arg8 = parm8;				\
-  volatile __typeof__((parm9)) arg9 = parm9;				\
-  volatile __typeof__((parm10)) arg10 = parm10;				\
+  STAP_SDT_VOLATILE __typeof__((parm1)) arg1 = parm1;			\
+  STAP_SDT_VOLATILE __typeof__((parm2)) arg2 = parm2;			\
+  STAP_SDT_VOLATILE __typeof__((parm3)) arg3 = parm3;			\
+  STAP_SDT_VOLATILE __typeof__((parm4)) arg4 = parm4;			\
+  STAP_SDT_VOLATILE __typeof__((parm5)) arg5 = parm5;			\
+  STAP_SDT_VOLATILE __typeof__((parm6)) arg6 = parm6;			\
+  STAP_SDT_VOLATILE __typeof__((parm7)) arg7 = parm7;			\
+  STAP_SDT_VOLATILE __typeof__((parm8)) arg8 = parm8;			\
+  STAP_SDT_VOLATILE __typeof__((parm9)) arg9 = parm9;			\
+  STAP_SDT_VOLATILE __typeof__((parm10)) arg10 = parm10;			\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
   __asm__ volatile ("2:\n"						\
