@@ -28,15 +28,27 @@ static void _stp_stack_print_fallback(unsigned long stack, int verbose, int leve
 
 
 static void __stp_stack_print(struct pt_regs *regs, int verbose, int levels,
-                              struct task_struct *tsk)
+                              struct task_struct *tsk, struct uretprobe_instance *ri)
 {
 #ifdef STP_USE_DWARF_UNWINDER
+        int start_levels = levels;
 	// FIXME: large stack allocation
 	struct unwind_frame_info info;
 	arch_unw_init_frame_info(&info, regs);
 
 	while (levels && (tsk || !arch_unw_user_mode(&info))) {
 		int ret = unwind(&info, tsk);
+#ifdef CONFIG_UTRACE
+                unsigned long maybe_pc = 0;
+                if (ri) {
+                        maybe_pc = uprobe_get_pc(ri, UNW_PC(&info),
+                                                 UNW_SP(&info));
+                        if (!maybe_pc)
+                                printk("SYSTEMTAP ERROR: uprobe_get_return returned 0\n");
+                        else
+                                UNW_PC(&info) = maybe_pc;
+                }
+#endif
 		dbug_unwind(1, "ret=%d PC=%lx SP=%lx\n", ret, UNW_PC(&info), UNW_SP(&info));
 		if (ret == 0) {
 			_stp_func_print(UNW_PC(&info), verbose, 1, tsk);
