@@ -94,24 +94,13 @@ static void print_stack_address(void *data, unsigned long addr, int reliable)
                 _stp_func_print(addr, sdata->verbose, 0, NULL);
 }
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,32)
-static unsigned long
-walk_context_stack(struct thread_info *tinfo,
-                   unsigned long *stack, unsigned long bp,
-                   const struct stacktrace_ops *ops, void *data,
-                   unsigned long *end, int *graph)
-{
-	return 0 ;
-}
-#endif
-
 static const struct stacktrace_ops print_stack_ops = {
 	.warning = print_stack_warning,
 	.warning_symbol = print_stack_warning_symbol,
 	.stack = print_stack_stack,
 	.address = print_stack_address,
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,32)
-	.walk_stack = walk_context_stack,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+	.walk_stack = print_context_stack,
 #endif
 };
 
@@ -197,7 +186,7 @@ static void _stp_stack_snprint(char *str, int size, struct pt_regs *regs, int ve
 
 #endif /* CONFIG_KPROBES */
 
-void _stp_stack_print_tsk(struct task_struct *tsk, int verbose, int levels)
+static void _stp_stack_print_tsk(struct task_struct *tsk, int verbose, int levels)
 {
 #if defined(STAPCONF_KERNEL_STACKTRACE)
         int i;
@@ -224,7 +213,7 @@ void _stp_stack_print_tsk(struct task_struct *tsk, int verbose, int levels)
  * @param tsk A pointer to the task_struct
  * @returns void
  */
-void _stp_stack_snprint_tsk(char *str, int size, struct task_struct *tsk, int verbose, int levels)
+static void _stp_stack_snprint_tsk(char *str, int size, struct task_struct *tsk, int verbose, int levels)
 {
 	_stp_pbuf *pb = per_cpu_ptr(Stp_pbuf, smp_processor_id());
 	_stp_print_flush();
