@@ -19,6 +19,7 @@
 #include <linux/namei.h>
 #include <linux/workqueue.h>
 #include <linux/delay.h>
+#include <linux/mutex.h>
 
 static int _stp_exit_flag = 0;
 
@@ -30,6 +31,9 @@ static int _stp_ctl_attached = 0;
 
 static pid_t _stp_target = 0;
 static int _stp_probes_started = 0;
+static int _stp_exit_called = 0;
+static DEFINE_MUTEX(_stp_transport_mutex);
+
 
 // For now, disable transport version 3 (unless STP_USE_RING_BUFFER is
 // defined).
@@ -81,22 +85,26 @@ static struct workqueue_struct *_stp_wq;
 
 static void _stp_handle_start(struct _stp_msg_start *st)
 {
-	dbug_trans(1, "stp_handle_start\n");
+	mutex_lock(&_stp_transport_mutex);
+	if (!_stp_exit_called) {
+		dbug_trans(1, "stp_handle_start\n");
 
 #ifdef STAPCONF_VM_AREA
-        { /* PR9740: workaround for kernel valloc bug. */
-                void *dummy;
-                dummy = alloc_vm_area (PAGE_SIZE);
-                free_vm_area (dummy);
-        }
+		{ /* PR9740: workaround for kernel valloc bug. */
+			void *dummy;
+			dummy = alloc_vm_area (PAGE_SIZE);
+			free_vm_area (dummy);
+		}
 #endif
 
-	_stp_target = st->target;
-	st->res = probe_start();
-	if (st->res >= 0)
-		_stp_probes_started = 1;
+		_stp_target = st->target;
+		st->res = probe_start();
+		if (st->res >= 0)
+			_stp_probes_started = 1;
 
-	_stp_ctl_send(STP_START, st, sizeof(*st));
+		_stp_ctl_send(STP_START, st, sizeof(*st));
+	}
+	mutex_unlock(&_stp_transport_mutex);
 }
 
 /* common cleanup code. */
@@ -106,8 +114,7 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 /* when someone does /sbin/rmmod on a loaded systemtap module. */
 static void _stp_cleanup_and_exit(int send_exit)
 {
-	static int _stp_exit_called = 0;
-
+	mutex_lock(&_stp_transport_mutex);
 	if (!_stp_exit_called) {
 		int failures;
 
@@ -135,6 +142,7 @@ static void _stp_cleanup_and_exit(int send_exit)
 			_stp_ctl_send(STP_EXIT, NULL, 0);
 		dbug_trans(1, "done with ctl_send STP_EXIT\n");
 	}
+	mutex_unlock(&_stp_transport_mutex);
 }
 
 static void _stp_request_exit(void)
@@ -293,7 +301,7 @@ err0:
 
 static inline void _stp_lock_inode(struct inode *inode)
 {
-#ifdef DEFINE_MUTEX
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
 	mutex_lock(&inode->i_mutex);
 #else
 	down(&inode->i_sem);
@@ -302,7 +310,7 @@ static inline void _stp_lock_inode(struct inode *inode)
 
 static inline void _stp_unlock_inode(struct inode *inode)
 {
-#ifdef DEFINE_MUTEX
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
 	mutex_unlock(&inode->i_mutex);
 #else
 	up(&inode->i_sem);
@@ -314,6 +322,7 @@ static struct dentry *_stp_lockfile = NULL;
 static int _stp_lock_transport_dir(void)
 {
 	int numtries = 0;
+
 #if STP_TRANSPORT_VERSION == 1
 	while ((_stp_lockfile = relayfs_create_dir("systemtap_lock", NULL)) == NULL) {
 #else
@@ -341,7 +350,10 @@ static void _stp_unlock_transport_dir(void)
 static struct dentry *__stp_root_dir = NULL;
 
 /* _stp_get_root_dir() - creates root directory or returns
- * a pointer to it if it already exists. */
+ * a pointer to it if it already exists.
+ *
+ * The caller *must* lock the transport directory.
+ */
 
 static struct dentry *_stp_get_root_dir(void)
 {
@@ -367,10 +379,6 @@ static struct dentry *_stp_get_root_dir(void)
 	}
 #endif
 
-	if (!_stp_lock_transport_dir()) {
-		errk("Couldn't lock transport directory.\n");
-		return NULL;
-	}
 #if STP_TRANSPORT_VERSION == 1
 	__stp_root_dir = relayfs_create_dir(name, NULL);
 #else
@@ -398,17 +406,17 @@ static struct dentry *_stp_get_root_dir(void)
 		 -PTR_ERR(__stp_root_dir));
 	}
 
-	_stp_unlock_transport_dir();
 	return __stp_root_dir;
 }
 
+/* _stp_remove_root_dir() - removes root directory (if empty)
+ *
+ * The caller *must* lock the transport directory.
+ */
+
 static void _stp_remove_root_dir(void)
 {
 	if (__stp_root_dir) {
-		if (!_stp_lock_transport_dir()) {
-			errk("Unable to lock transport directory.\n");
-			return;
-		}
 		if (simple_empty(__stp_root_dir)) {
 #if STP_TRANSPORT_VERSION == 1
 			relayfs_remove_dir(__stp_root_dir);
@@ -416,7 +424,6 @@ static void _stp_remove_root_dir(void)
 			debugfs_remove(__stp_root_dir);
 #endif
 		}
-		_stp_unlock_transport_dir();
 		__stp_root_dir = NULL;
 	}
 }
@@ -436,9 +443,16 @@ static int _stp_transport_fs_init(const char *module_name)
 	if (module_name == NULL)
 		return -1;
 
-	root_dir = _stp_get_root_dir();
-	if (root_dir == NULL)
+	if (!_stp_lock_transport_dir()) {
+		errk("Couldn't lock transport directory.\n");
 		return -1;
+	}
+
+	root_dir = _stp_get_root_dir();
+	if (root_dir == NULL) {
+		_stp_unlock_transport_dir();
+		return -1;
+	}
 
 #if STP_TRANSPORT_VERSION == 1
         __stp_module_dir = relayfs_create_dir(module_name, root_dir);
@@ -449,19 +463,23 @@ static int _stp_transport_fs_init(const char *module_name)
 		errk("Could not create module directory \"%s\"\n",
 		     module_name);
 		_stp_remove_root_dir();
+		_stp_unlock_transport_dir();
 		return -1;
 	}
 	else if (IS_ERR(__stp_module_dir)) {
 		errk("Could not create module directory \"%s\", error %ld\n",
 		     module_name, -PTR_ERR(__stp_module_dir));
 		_stp_remove_root_dir();
+		_stp_unlock_transport_dir();
 		return -1;
 	}
 
 	if (_stp_transport_data_fs_init() != 0) {
 		_stp_remove_root_dir();
+		_stp_unlock_transport_dir();
 		return -1;
 	}
+	_stp_unlock_transport_dir();
 	dbug_trans(1, "returning 0\n");
 	return 0;
 }
@@ -473,15 +491,21 @@ static void _stp_transport_fs_close(void)
 	_stp_transport_data_fs_close();
 
 	if (__stp_module_dir) {
+		if (!_stp_lock_transport_dir()) {
+			errk("Couldn't lock transport directory.\n");
+			return;
+		}
+
 #if STP_TRANSPORT_VERSION == 1
 		relayfs_remove_dir(__stp_module_dir);
 #else
 		debugfs_remove(__stp_module_dir);
 #endif
 		__stp_module_dir = NULL;
-	}
 
-	_stp_remove_root_dir();
+		_stp_remove_root_dir();
+		_stp_unlock_transport_dir();
+	}
 }
 
 
