@@ -652,8 +652,18 @@ dont_add:
 	return NULL;
 }
 
-/* Runs with uproc_mutex held; returns with uproc->rwsem write-locked. */
-static struct uprobe_process *uprobe_mk_process(struct task_struct *p)
+/*
+ * Create a per process uproc struct.
+ * at_fork: indicates uprobe_mk_process is called from
+ * a fork context of a probe process. refer uprobe_fork_uproc
+ * for more details.
+ *
+ * Runs with uproc_mutex held;
+ * Returns with uproc->rwsem write-locked when not called
+ * from fork context.
+ */
+static struct uprobe_process *uprobe_mk_process(struct task_struct *p,
+						bool at_fork)
 {
 	struct uprobe_process *uproc;
 	struct uprobe_task *utask;
@@ -668,7 +678,9 @@ static struct uprobe_process *uprobe_mk_process(struct task_struct *p)
 	/* Initialize fields */
 	atomic_set(&uproc->refcount, 1);
 	init_rwsem(&uproc->rwsem);
-	down_write(&uproc->rwsem);
+	if (!at_fork)
+		/* not called from fork context. */
+		down_write(&uproc->rwsem);
 	init_waitqueue_head(&uproc->waitq);
 	for (i = 0; i < UPROBE_TABLE_SIZE; i++)
 		INIT_HLIST_HEAD(&uproc->uprobe_table[i]);
@@ -923,7 +935,7 @@ int register_uprobe(struct uprobe *u)
 			unlock_uproc_table();
 			goto fail_tsk;
 		}
-		uproc = uprobe_mk_process(p);
+		uproc = uprobe_mk_process(p, 0);
 		if (IS_ERR(uproc)) {
 			ret = (int) PTR_ERR(uproc);
 			unlock_uproc_table();
@@ -2206,13 +2218,12 @@ static int uprobe_fork_uproc(struct uprobe_process *parent_uproc,
 
 	if (!try_module_get(THIS_MODULE))
 		return -ENOSYS;
-	child_uproc = uprobe_mk_process(child_tsk);
+	child_uproc = uprobe_mk_process(child_tsk, 1);
 	if (IS_ERR(child_uproc)) {
 		ret = (int) PTR_ERR(child_uproc);
 		module_put(THIS_MODULE);
 		return ret;
 	}
-	/* child_uproc is write-locked and ref-counted at this point. */
 
 	mutex_lock(&child_uproc->ssol_area.setup_mutex);
 	uprobe_init_ssol(child_uproc, child_tsk,
@@ -2228,7 +2239,6 @@ static int uprobe_fork_uproc(struct uprobe_process *parent_uproc,
 			&uproc_table[hash_long(child_uproc->tgid,
 			UPROBE_HASH_BITS)]);
 
-	up_write(&child_uproc->rwsem);
 	uprobe_decref_process(child_uproc);
 	return ret;
 }
@@ -2586,6 +2596,87 @@ static void uretprobe_set_trampoline(struct uprobe_process *uproc,
 	}
 }
 
+static inline unsigned long lookup_uretprobe(struct hlist_node *r,
+					     struct uprobe_process *uproc,
+					     unsigned long pc,
+					     unsigned long sp)
+{
+	struct uretprobe_instance *ret_inst;
+	unsigned long trampoline_addr;
+	
+	if (IS_ERR(uproc->uretprobe_trampoline_addr))
+	  return pc;
+	trampoline_addr = (unsigned long)uproc->uretprobe_trampoline_addr;
+	if (pc != trampoline_addr)
+		return pc;
+	hlist_for_each_entry_from(ret_inst, r, hlist) {
+		if (ret_inst->ret_addr == trampoline_addr)
+			continue;
+		/* First handler with a stack pointer lower than the
+		   address (or equal) must be the one. */
+		if (ret_inst->sp == sp || compare_stack_ptrs(ret_inst->sp, sp))
+			return ret_inst->ret_addr;
+	}
+	printk(KERN_ERR "Original return address for trampoline not found at "
+	       "0x%lx pid/tgid=%d/%d\n", sp, current->pid, current->tgid);
+	return 0;
+
+}
+
+unsigned long uprobe_get_pc(struct uretprobe_instance *ri, unsigned long pc,
+			unsigned long sp)
+{
+	struct uretprobe *rp;
+	struct uprobe_kimg *uk;
+	struct uprobe_task *utask;
+	struct uprobe_process *uproc;
+	struct hlist_node *r;
+
+	if (!ri)
+		return 0;
+	if (ri == GET_PC_URETPROBE_NONE) {
+		utask = uprobe_find_utask(current);
+		if (!utask)
+			return 0;
+		uproc = utask->uproc;
+		r = utask->uretprobe_instances.first;
+	} else {
+		rp = ri->rp;
+		uk = (struct uprobe_kimg *)rp->u.kdata;
+		if (!uk)
+			return 0;
+		uproc = uk->ppt->uproc;
+		r = &ri->hlist;		
+	}
+	return lookup_uretprobe(r, uproc, pc, sp);
+}
+
+EXPORT_SYMBOL_GPL(uprobe_get_pc);
+
+unsigned long uprobe_get_pc_task(struct task_struct *task, unsigned long pc,
+				 unsigned long sp)
+{
+	struct uprobe_task *utask;
+	struct uprobe_process *uproc;
+	unsigned long result;
+		
+	utask = uprobe_find_utask(task);
+	if (!utask) {
+		return pc;
+	} else if (current == task && utask->active_probe) {
+		/* everything's locked. */
+		return uprobe_get_pc(GET_PC_URETPROBE_NONE, pc, sp);
+	}
+	uproc = utask->uproc;
+	down_read(&uproc->rwsem);
+	result = lookup_uretprobe(utask->uretprobe_instances.first, uproc, pc,
+				  sp);
+	up_read(&uproc->rwsem);
+	return result;
+}
+
+EXPORT_SYMBOL_GPL(uprobe_get_pc_task);
+
 #else	/* ! CONFIG_URETPROBES */
 
 static void uretprobe_handle_entry(struct uprobe *u, struct pt_regs *regs,
