@@ -726,8 +726,18 @@ static struct pid *find_next_thread_to_add(struct uprobe_process *uproc,
 	return pid;
 }
 
-/* Runs with uproc_mutex held; returns with uproc->rwsem write-locked. */
-static struct uprobe_process *uprobe_mk_process(struct pid *tg_leader)
+/*
+ * Create a per process uproc struct.
+ * at_fork: indicates uprobe_mk_process is called from
+ * a fork context of a probe process. refer uprobe_fork_uproc
+ * for more details.
+ *
+ * Runs with uproc_mutex held;
+ * Returns with uproc->rwsem write-locked when not called
+ * from fork context.
+ */
+static struct uprobe_process *uprobe_mk_process(struct pid *tg_leader,
+						bool at_fork)
 {
 	struct uprobe_process *uproc;
 	struct uprobe_task *utask;
@@ -742,7 +752,9 @@ static struct uprobe_process *uprobe_mk_process(struct pid *tg_leader)
 	/* Initialize fields */
 	atomic_set(&uproc->refcount, 1);
 	init_rwsem(&uproc->rwsem);
-	down_write(&uproc->rwsem);
+	if (!at_fork)
+		/* not called from fork context. */
+		down_write(&uproc->rwsem);
 	init_waitqueue_head(&uproc->waitq);
 	for (i = 0; i < UPROBE_TABLE_SIZE; i++)
 		INIT_HLIST_HEAD(&uproc->uprobe_table[i]);
@@ -1019,7 +1031,7 @@ int register_uprobe(struct uprobe *u)
 			unlock_uproc_table();
 			goto fail_tsk;
 		}
-		uproc = uprobe_mk_process(p);
+		uproc = uprobe_mk_process(p, 0);
 		if (IS_ERR(uproc)) {
 			ret = (int) PTR_ERR(uproc);
 			unlock_uproc_table();
@@ -1869,7 +1881,9 @@ static void uprobe_inject_delayed_signals(struct list_head *delayed_signals)
  */
 static u32 uprobe_report_signal(u32 action,
 				struct utrace_attached_engine *engine,
+#if !(defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216))
 				struct task_struct *tsk,
+#endif
 				struct pt_regs *regs,
 				siginfo_t *info,
 				const struct k_sigaction *orig_ka,
@@ -2117,9 +2131,15 @@ static int utask_quiesce_pending_sigtrap(struct uprobe_task *utask)
  * insertions or removals pending.  If we're the last thread in this
  * process to quiesce, do the insertion(s) and/or removal(s).
  */
-static u32 uprobe_report_quiesce(enum utrace_resume_action action,
+static u32 uprobe_report_quiesce(
+#if defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216)
+				u32 action,
+				struct utrace_attached_engine *engine,
+#else
+				enum utrace_resume_action action,
 				struct utrace_attached_engine *engine,
 				struct task_struct *tsk,
+#endif
 				unsigned long event)
 {
 	struct uprobe_task *utask;
@@ -2128,7 +2148,9 @@ static u32 uprobe_report_quiesce(enum utrace_resume_action action,
 
 	utask = (struct uprobe_task *)rcu_dereference(engine->data);
 	BUG_ON(!utask);
+#if !(defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216))
 	BUG_ON(tsk != current);	// guaranteed by utrace 2008
+#endif
 
 	if (utask->state == UPTASK_SSTEP)
 		/*
@@ -2231,8 +2253,14 @@ static void uprobe_cleanup_process(struct uprobe_process *uproc)
  */
 static u32 uprobe_report_exit(enum utrace_resume_action action,
 			struct utrace_attached_engine *engine,
-			struct task_struct *tsk, long orig_code, long *code)
+#if !(defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216))
+			struct task_struct *tsk,
+#endif
+			long orig_code, long *code)
 {
+#if defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216)
+	struct task_struct *tsk = current;
+#endif
 	struct uprobe_task *utask;
 	struct uprobe_process *uproc;
 	struct uprobe_probept *ppt;
@@ -2398,14 +2426,13 @@ static int uprobe_fork_uproc(struct uprobe_process *parent_uproc,
 		module_put(THIS_MODULE);
 		return -ESRCH;
 	}
-	child_uproc = uprobe_mk_process(child_pid);
+	child_uproc = uprobe_mk_process(child_pid, 1);
 	put_pid(child_pid);
 	if (IS_ERR(child_uproc)) {
 		ret = (int) PTR_ERR(child_uproc);
 		module_put(THIS_MODULE);
 		return ret;
 	}
-	/* child_uproc is write-locked and ref-counted at this point. */
 
 	mutex_lock(&child_uproc->ssol_area.setup_mutex);
 	uprobe_init_ssol(child_uproc, child_tsk,
@@ -2420,7 +2447,6 @@ static int uprobe_fork_uproc(struct uprobe_process *parent_uproc,
 	hlist_add_head(&child_uproc->hlist,
 			&uproc_table[hash_ptr(child_pid, UPROBE_HASH_BITS)]);
 
-	up_write(&child_uproc->rwsem);
 	uprobe_decref_process(child_uproc);
 	return ret;
 }
@@ -2439,10 +2465,15 @@ static int uprobe_fork_uproc(struct uprobe_process *parent_uproc,
  */
 static u32 uprobe_report_clone(enum utrace_resume_action action,
 				struct utrace_attached_engine *engine,
+#if !(defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216))
 				struct task_struct *parent,
+#endif
 				unsigned long clone_flags,
 				struct task_struct *child)
 {
+#if defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216)
+	struct task_struct *parent = current;
+#endif
 	int len;
 	struct uprobe_process *uproc;
 	struct uprobe_task *ptask, *ctask;
@@ -2544,9 +2575,15 @@ done:
  *		- We have to free up uprobe resources associated with
  *		  this process.
  */
-static u32 uprobe_report_exec(enum utrace_resume_action action,
+static u32 uprobe_report_exec(
+#if defined(UTRACE_API_VERSION) && (UTRACE_API_VERSION >= 20091216)
+				u32 action,
 				struct utrace_attached_engine *engine,
-				struct task_struct *tsk,
+#else
+				enum utrace_resume_action action,
+				struct utrace_attached_engine *engine,
+				struct task_struct *parent,
+#endif
 				const struct linux_binfmt *fmt,
 				const struct linux_binprm *bprm,
 				struct pt_regs *regs)
@@ -2800,6 +2837,86 @@ static void uretprobe_set_trampoline(struct uprobe_process *uproc,
 	}
 }
 
+static inline unsigned long lookup_uretprobe(struct hlist_node *r,
+					     struct uprobe_process *uproc,
+					     unsigned long pc,
+					     unsigned long sp)
+{
+	struct uretprobe_instance *ret_inst;
+	unsigned long trampoline_addr;
+	
+	if (IS_ERR(uproc->uretprobe_trampoline_addr))
+	  return pc;
+	trampoline_addr = (unsigned long)uproc->uretprobe_trampoline_addr;
+	if (pc != trampoline_addr)
+		return pc;
+	hlist_for_each_entry_from(ret_inst, r, hlist) {
+		if (ret_inst->ret_addr == trampoline_addr)
+			continue;
+		/* First handler with a stack pointer lower than the
+		   address (or equal) must be the one. */
+		if (ret_inst->sp == sp || compare_stack_ptrs(ret_inst->sp, sp))
+			return ret_inst->ret_addr;
+	}
+	printk(KERN_ERR "Original return address for trampoline not found at "
+	       "0x%lx pid/tgid=%d/%d\n", sp, current->pid, current->tgid);
+	return 0;
+
+}
+
+unsigned long uprobe_get_pc(struct uretprobe_instance *ri, unsigned long pc,
+			unsigned long sp)
+{
+	struct uretprobe *rp;
+	struct uprobe_kimg *uk;
+	struct uprobe_task *utask;
+	struct uprobe_process *uproc;
+	struct hlist_node *r;
+
+	if (!ri)
+		return 0;
+	if (ri == GET_PC_URETPROBE_NONE) {
+		utask = uprobe_find_utask(current);
+		if (!utask)
+			return 0;
+		uproc = utask->uproc;
+		r = utask->uretprobe_instances.first;
+	} else {
+		rp = ri->rp;
+		uk = (struct uprobe_kimg *)rp->u.kdata;
+		if (!uk)
+			return 0;
+		uproc = uk->ppt->uproc;
+		r = &ri->hlist;		
+	}
+	return lookup_uretprobe(r, uproc, pc, sp);
+}
+
+EXPORT_SYMBOL_GPL(uprobe_get_pc);
+
+unsigned long uprobe_get_pc_task(struct task_struct *task, unsigned long pc,
+				 unsigned long sp)
+{
+	struct uprobe_task *utask;
+	struct uprobe_process *uproc;
+	unsigned long result;
+		
+	utask = uprobe_find_utask(task);
+	if (!utask) {
+		return pc;
+	} else if (current == task && utask->active_probe) {
+		/* everything's locked. */
+		return uprobe_get_pc(GET_PC_URETPROBE_NONE, pc, sp);
+	}
+	uproc = utask->uproc;
+	down_read(&uproc->rwsem);
+	result = lookup_uretprobe(utask->uretprobe_instances.first, uproc, pc,
+				  sp);
+	up_read(&uproc->rwsem);
+	return result;
+}
+
+EXPORT_SYMBOL_GPL(uprobe_get_pc_task);
 #else	/* ! CONFIG_URETPROBES */
 
 static void uretprobe_handle_entry(struct uprobe *u, struct pt_regs *regs,
