@@ -1,5 +1,5 @@
 // parse tree functions
-// Copyright (C) 2005-2009 Red Hat Inc.
+// Copyright (C) 2005-2010 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -42,6 +42,18 @@ statement::statement ():
 }
 
 
+statement::statement (const token* tok):
+  tok (tok)
+{
+}
+
+
+null_statement::null_statement (const token* tok):
+  statement(tok)
+{
+}
+
+
 statement::~statement ()
 {
 }
@@ -152,7 +164,7 @@ vardecl::compatible_arity (int a)
 
 
 functiondecl::functiondecl ():
-  body (0)
+  body (0), synthetic (false)
 {
 }
 
@@ -211,6 +223,14 @@ target_symbol::assert_no_components(const std::string& tapset)
 }
 
 
+void target_symbol::chain (semantic_error *e)
+{
+  assert (e->chain == 0);
+  e->chain = this->saved_conversion_error;
+  this->saved_conversion_error = e;
+}
+
+
 // ------------------------------------------------------------------------
 // parse tree printing
 
@@ -332,6 +352,12 @@ void cast_op::print (ostream& o) const
 }
 
 
+void defined_op::print (ostream& o) const
+{
+  o << "@defined(" << *operand << ")";
+}
+
+
 void vardecl::print (ostream& o) const
 {
   o << name;
@@ -877,6 +903,18 @@ block::block (statement* car, statement* cdr)
 
 
 
+void try_block::print (ostream& o) const
+{
+  o << "try {" << endl;
+  if (try_block) o << *try_block << endl;
+  o << "} catch ";
+  if (catch_error_var) o << "(" << *catch_error_var << ") ";
+  o << "{" << endl;
+  if (catch_block) o << *catch_block << endl;
+  o << "}" << endl;
+}
+
+
 void for_loop::print (ostream& o) const
 {
   o << "for (";
@@ -1113,6 +1151,13 @@ block::visit (visitor* u)
 }
 
 
+void
+try_block::visit (visitor* u)
+{
+  u->visit_try_block (this);
+}
+
+
 void
 embeddedcode::visit (visitor* u)
 {
@@ -1300,6 +1345,14 @@ cast_op::visit (visitor* u)
   u->visit_cast_op(this);
 }
 
+
+void
+defined_op::visit (visitor* u)
+{
+  u->visit_defined_op(this);
+}
+
+
 void
 arrayindex::visit (visitor* u)
 {
@@ -1490,6 +1543,17 @@ traversing_visitor::visit_block (block* s)
     s->statements[i]->visit (this);
 }
 
+void
+traversing_visitor::visit_try_block (try_block* s)
+{
+  if (s->try_block)
+    s->try_block->visit (this);
+  if (s->catch_error_var)
+    s->catch_error_var->visit (this);
+  if (s->catch_block)
+    s->catch_block->visit (this);
+}
+
 void
 traversing_visitor::visit_embeddedcode (embeddedcode*)
 {
@@ -1674,6 +1738,13 @@ traversing_visitor::visit_cast_op (cast_op* e)
   e->visit_components (this);
 }
 
+void
+traversing_visitor::visit_defined_op (defined_op* e)
+{
+  e->operand->visit (this);
+}
+
+
 void
 traversing_visitor::visit_arrayindex (arrayindex* e)
 {
@@ -1736,6 +1807,21 @@ functioncall_traversing_visitor::visit_functioncall (functioncall* e)
 }
 
 
+void
+varuse_collecting_visitor::visit_try_block (try_block *s)
+{
+  if (s->try_block)
+    s->try_block->visit (this);
+  if (s->catch_error_var)
+    written.insert (s->catch_error_var->referent);
+  if (s->catch_block)
+    s->catch_block->visit (this);
+
+  // NB: don't functioncall_traversing_visitor::visit_try_block (s);
+  // since that would count s->catch_error_var as a read also.
+}
+
+
 void
 varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 {
@@ -1788,6 +1874,14 @@ varuse_collecting_visitor::visit_cast_op (cast_op *e)
   functioncall_traversing_visitor::visit_cast_op (e);
 }
 
+void
+varuse_collecting_visitor::visit_defined_op (defined_op *e)
+{
+  // XXX
+  functioncall_traversing_visitor::visit_defined_op (e);
+}
+
+
 void
 varuse_collecting_visitor::visit_print_format (print_format* e)
 {
@@ -1897,19 +1991,21 @@ varuse_collecting_visitor::visit_arrayindex (arrayindex *e)
 void
 varuse_collecting_visitor::visit_pre_crement (pre_crement *e)
 {
-  expression* last_lrvalue = current_lrvalue;
-  current_lrvalue = e->operand; // leave a mark for ::visit_symbol
+  // PR6954: regard as pure writes
+  expression* last_lvalue = current_lvalue;
+  current_lvalue = e->operand; // leave a mark for ::visit_symbol
   functioncall_traversing_visitor::visit_pre_crement (e);
-  current_lrvalue = last_lrvalue;
+  current_lvalue = last_lvalue;
 }
 
 void
 varuse_collecting_visitor::visit_post_crement (post_crement *e)
 {
-  expression* last_lrvalue = current_lrvalue;
-  current_lrvalue = e->operand; // leave a mark for ::visit_symbol
+  // PR6954: regard as pure writes
+  expression* last_lvalue = current_lvalue;
+  current_lvalue = e->operand; // leave a mark for ::visit_symbol
   functioncall_traversing_visitor::visit_post_crement (e);
-  current_lrvalue = last_lrvalue;
+  current_lvalue = last_lvalue;
 }
 
 void
@@ -2015,6 +2111,13 @@ throwing_visitor::visit_block (block* s)
   throwone (s->tok);
 }
 
+void
+throwing_visitor::visit_try_block (try_block* s)
+{
+  throwone (s->tok);
+}
+
+
 void
 throwing_visitor::visit_embeddedcode (embeddedcode* s)
 {
@@ -2178,6 +2281,13 @@ throwing_visitor::visit_cast_op (cast_op* e)
   throwone (e->tok);
 }
 
+void
+throwing_visitor::visit_defined_op (defined_op* e)
+{
+  throwone (e->tok);
+}
+
+
 void
 throwing_visitor::visit_arrayindex (arrayindex* e)
 {
@@ -2220,6 +2330,15 @@ update_visitor::visit_block (block* s)
   provide (s);
 }
 
+void
+update_visitor::visit_try_block (try_block* s)
+{
+  replace (s->try_block);
+  replace (s->catch_error_var);
+  replace (s->catch_block);
+  provide (s);
+}
+
 void
 update_visitor::visit_embeddedcode (embeddedcode* s)
 {
@@ -2420,6 +2539,13 @@ update_visitor::visit_cast_op (cast_op* e)
   provide (e);
 }
 
+void
+update_visitor::visit_defined_op (defined_op* e)
+{
+  replace (e->operand);
+  provide (e);
+}
+
 void
 update_visitor::visit_arrayindex (arrayindex* e)
 {
@@ -2490,6 +2616,12 @@ deep_copy_visitor::visit_block (block* s)
   update_visitor::visit_block(new block(*s));
 }
 
+void
+deep_copy_visitor::visit_try_block (try_block* s)
+{
+  update_visitor::visit_try_block(new try_block(*s));
+}
+
 void
 deep_copy_visitor::visit_embeddedcode (embeddedcode* s)
 {
@@ -2657,6 +2789,12 @@ deep_copy_visitor::visit_cast_op (cast_op* e)
   update_visitor::visit_cast_op(new cast_op(*e));
 }
 
+void
+deep_copy_visitor::visit_defined_op (defined_op* e)
+{
+  update_visitor::visit_defined_op(new defined_op(*e));
+}
+
 void
 deep_copy_visitor::visit_arrayindex (arrayindex* e)
 {
