@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2009 Red Hat Inc.
+// Copyright (C) 2005-2010 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -254,9 +254,10 @@ struct target_symbol: public symbol
   bool addressof;
   std::string base_name;
   std::vector<component> components;
-  std::string probe_context_var;
-  semantic_error* saved_conversion_error;
+  std::string probe_context_var; // NB: this being set implies that target_symbol is *resolved*
+  semantic_error* saved_conversion_error; // hand-made linked list
   target_symbol(): addressof(false), saved_conversion_error (0) {}
+  void chain (semantic_error* e);
   void print (std::ostream& o) const;
   void visit (visitor* u);
   void visit_components (visitor* u);
@@ -276,6 +277,14 @@ struct cast_op: public target_symbol
 };
 
 
+struct defined_op: public expression
+{
+  target_symbol *operand;
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
+
+
 struct arrayindex: public expression
 {
   std::vector<expression*> indexes;
@@ -477,6 +486,7 @@ struct functiondecl: public symboldecl
   std::vector<vardecl*> locals;
   std::vector<vardecl*> unused_locals;
   statement* body;
+  bool synthetic;
   functiondecl ();
   void print (std::ostream& o) const;
   void printsig (std::ostream& o) const;
@@ -492,6 +502,7 @@ struct statement
   virtual void visit (visitor* u) = 0;
   const token* tok;
   statement ();
+  statement (const token* tok);
   virtual ~statement ();
 };
 
@@ -516,6 +527,16 @@ struct block: public statement
 };
 
 
+struct try_block: public statement
+{
+  statement* try_block; // may be 0
+  statement* catch_block; // may be 0
+  symbol* catch_error_var; // may be 0
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
+
+
 struct expr_statement;
 struct for_loop: public statement
 {
@@ -547,6 +568,7 @@ struct null_statement: public statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  null_statement (const token* tok);
 };
 
 
@@ -659,6 +681,7 @@ struct probe
   virtual void printsig (std::ostream &o) const;
   virtual void collect_derivation_chain (std::vector<probe*> &probes_list);
   virtual const probe_alias *get_alias () const { return 0; }
+  virtual probe* create_alias(probe_point* l, probe_point* a);
   virtual probe* basest () { return this; }
   virtual ~probe() {}
   bool privileged;
@@ -686,6 +709,7 @@ struct visitor
 
   virtual ~visitor () {}
   virtual void visit_block (block *s) = 0;
+  virtual void visit_try_block (try_block *s) = 0;
   virtual void visit_embeddedcode (embeddedcode *s) = 0;
   virtual void visit_null_statement (null_statement *s) = 0;
   virtual void visit_expr_statement (expr_statement *s) = 0;
@@ -718,6 +742,7 @@ struct visitor
   virtual void visit_stat_op (stat_op* e) = 0;
   virtual void visit_hist_op (hist_op* e) = 0;
   virtual void visit_cast_op (cast_op* e) = 0;
+  virtual void visit_defined_op (defined_op* e) = 0;
 };
 
 
@@ -727,6 +752,7 @@ struct visitor
 struct traversing_visitor: public visitor
 {
   void visit_block (block *s);
+  void visit_try_block (try_block *s);
   void visit_embeddedcode (embeddedcode *s);
   void visit_null_statement (null_statement *s);
   void visit_expr_statement (expr_statement *s);
@@ -759,6 +785,7 @@ struct traversing_visitor: public visitor
   void visit_stat_op (stat_op* e);
   void visit_hist_op (hist_op* e);
   void visit_cast_op (cast_op* e);
+  void visit_defined_op (defined_op* e);
 };
 
 
@@ -791,6 +818,7 @@ struct varuse_collecting_visitor: public functioncall_traversing_visitor
     current_lvalue(0),
     current_lrvalue(0) {}
   void visit_embeddedcode (embeddedcode *s);
+  void visit_try_block (try_block *s);
   void visit_delete_statement (delete_statement *s);
   void visit_print_format (print_format *e);
   void visit_assignment (assignment *e);
@@ -801,6 +829,7 @@ struct varuse_collecting_visitor: public functioncall_traversing_visitor
   void visit_post_crement (post_crement *e);
   void visit_foreach_loop (foreach_loop *s);
   void visit_cast_op (cast_op* e);
+  void visit_defined_op (defined_op* e);
 
   bool side_effect_free ();
   bool side_effect_free_wrt (const std::set<vardecl*>& vars);
@@ -819,6 +848,7 @@ struct throwing_visitor: public visitor
   virtual void throwone (const token* t);
 
   void visit_block (block *s);
+  void visit_try_block (try_block *s);
   void visit_embeddedcode (embeddedcode *s);
   void visit_null_statement (null_statement *s);
   void visit_expr_statement (expr_statement *s);
@@ -851,6 +881,7 @@ struct throwing_visitor: public visitor
   void visit_stat_op (stat_op* e);
   void visit_hist_op (hist_op* e);
   void visit_cast_op (cast_op* e);
+  void visit_defined_op (defined_op* e);
 };
 
 // A visitor similar to a traversing_visitor, but with the ability to rewrite
@@ -865,7 +896,7 @@ struct update_visitor: public visitor
       {
         src->visit(this);
         assert(!targets.empty());
-        dst = static_cast<T*>(targets.top());
+        dst = static_cast<T*>(targets.top()); // XXX: danger will robinson: not typesafe!
         targets.pop();
         assert(clearok || dst);
       }
@@ -874,7 +905,7 @@ struct update_visitor: public visitor
 
   template <typename T> void provide (T* src)
   {
-    targets.push(static_cast<void*>(src));
+    targets.push(static_cast<void*>(src)); // XXX: not typesafe!
   }
 
   template <typename T> void replace (T*& src, bool clearok=false)
@@ -885,6 +916,7 @@ struct update_visitor: public visitor
   virtual ~update_visitor() { assert(targets.empty()); }
 
   virtual void visit_block (block *s);
+  virtual void visit_try_block (try_block *s);
   virtual void visit_embeddedcode (embeddedcode *s);
   virtual void visit_null_statement (null_statement *s);
   virtual void visit_expr_statement (expr_statement *s);
@@ -917,6 +949,7 @@ struct update_visitor: public visitor
   virtual void visit_stat_op (stat_op* e);
   virtual void visit_hist_op (hist_op* e);
   virtual void visit_cast_op (cast_op* e);
+  virtual void visit_defined_op (defined_op* e);
 
 private:
   std::stack<void *> targets;
@@ -940,6 +973,7 @@ struct deep_copy_visitor: public update_visitor
   }
 
   virtual void visit_block (block *s);
+  virtual void visit_try_block (try_block *s);
   virtual void visit_embeddedcode (embeddedcode *s);
   virtual void visit_null_statement (null_statement *s);
   virtual void visit_expr_statement (expr_statement *s);
@@ -972,6 +1006,7 @@ struct deep_copy_visitor: public update_visitor
   virtual void visit_stat_op (stat_op* e);
   virtual void visit_hist_op (hist_op* e);
   virtual void visit_cast_op (cast_op* e);
+  virtual void visit_defined_op (defined_op* e);
 };
 
 #endif // STAPTREE_H
