@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005, 2009 Red Hat Inc.
+// Copyright (C) 2005-2010 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -22,9 +22,9 @@ void common_probe_entryfn_epilogue (translator_output* o, bool overload_processi
 void register_tapset_been(systemtap_session& sess);
 void register_tapset_itrace(systemtap_session& sess);
 void register_tapset_mark(systemtap_session& sess);
-void register_tapset_perfmon(systemtap_session& sess);
 void register_tapset_procfs(systemtap_session& sess);
 void register_tapset_timers(systemtap_session& sess);
+void register_tapset_perf(systemtap_session& sess);
 void register_tapset_utrace(systemtap_session& sess);
 
 
@@ -49,9 +49,13 @@ struct var_expanding_visitor: public update_visitor
 {
   static unsigned tick;
   std::stack<functioncall**> target_symbol_setter_functioncalls;
+  std::stack<defined_op*> defined_ops;
+  std::set<std::string> valid_ops;
+  std::string *op;
 
-  var_expanding_visitor() {}
+  var_expanding_visitor ();
   void visit_assignment (assignment* e);
+  void visit_defined_op (defined_op* e);
 };
 
 #endif // TAPSETS_H
