@@ -1,3 +1,4 @@
+#include <cstring>
 #include <string>
 #include <vector>
 #include <iostream>
@@ -21,7 +22,7 @@ const std::string cmdstr_quoted(const std::string& cmd);
 std::string git_revision(const std::string& path);
 int stap_system(int verbose, const std::string& command);
 int kill_stap_spawn(int sig);
-
+void assert_regexp_match (const std::string& name, const std::string& value, const std::string& re);
 
 // stringification generics
 
@@ -112,4 +113,23 @@ void delete_map(T& t)
 }
 
 
+// Returns whether a string starts with the given prefix
+inline bool
+startswith(const std::string & s, const char * prefix)
+{
+  return (s.compare(0, std::strlen(prefix), prefix) == 0);
+}
+
+
+// Returns whether a string ends with the given suffix
+inline bool
+endswith(const std::string & s, const char * suffix)
+{
+  size_t s_len = s.size(), suffix_len = std::strlen(suffix);
+  if (suffix_len > s_len)
+    return false;
+  return (s.compare(s_len - suffix_len, suffix_len, suffix) == 0);
+}
+
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
